#include "inputShaderHelperDX.h"

v2p vertexShader(vertexInfo input)
{
    v2p output;
    output.position = float4(input.position,1.0);
    output.texCoordY = input.texCoordY;
    output.texCoordUV = input.texCoordY;
    return output;
}

float4 pixelShader(v2p input) : SV_TARGET
{
    float4 yuvIn;

    float4 y0;
    float4 y1;

    float isOddUV;
    float textureWidth;
    float textureHeight;

    inputTex0.GetDimensions(textureWidth,textureHeight);

    isOddUV = floor(frac(input.texCoordY.x * 0.5) * 2.0);

    float2 normalizedTC=input.texCoordY/float2(textureWidth*2.,textureHeight);


    #ifdef YUYV
    y0 = float4(inputTex0.Sample(clearBorderSampler,normalizedTC)).argb;
    #endif

    #ifdef UYVY
    y0 = float4(inputTex0.Sample(clearBorderSampler,normalizedTC));
    #endif


    y1=y0;
    y1.g=y0.a;
    y0=lerp(y0,y1,isOddUV);

    y0+=stdbias;

    float4 outColor=float4(dot(yuv2R, y0),
        dot(yuv2G, y0),
        dot(yuv2B, y0),
        1.0);

#ifdef USE_COLOR_PROFILE
    float4 linearRGB=toLinear(outColor);

    float4 srgbLinear=float4(dot(profileToRGB_R, linearRGB),
        dot(profileToRGB_G, linearRGB),
        dot(profileToRGB_B, linearRGB),
        outColor.a);

    outColor=toGamma(srgbLinear);
#endif

#ifdef PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_MASK
    outColor.a*=maskTex.Sample(clearBorderSampler,normalizedTC).a;
#endif

#ifdef USE_END_COLOR
    outColor=mix(uniforms.endColor,outColor,uniforms.endOpacity);
#endif
    
    return outColor;
}
