
#include "metalShaderTypes.h"
#include "premultiply.h"

typedef struct
{
	float4 burnColor;
} fxVars;

vertex vertexOut vertexFunc(uint vertexID [[ vertex_id ]],
                            const device txShaderVers* in[[ buffer(0) ]],
                            constant txShaderUniforms& uniforms[[ buffer(3) ]],
                            constant fxVars& vars[[ buffer(2)]])
{
    vertexOut out;

    out.position=in[vertexID].pos*uniforms.worldViewProj;

    out.tex1=in[vertexID].tex1;
	out.tex2=in[vertexID].tex2;
	out.texMask=in[vertexID].texMask;
    return out;
}

fragment float4 fragmentFunc(vertexOut input [[stage_in]]
							 ,texture2d<half> inputTex0 [[ texture(0) ]]
							 ,texture2d<half> inputTex1 [[ texture(1) ]]
							 ,texture2d<half> inputTexMask [[ texture(2) ]]
							 ,constant txShaderUniforms& u[[ buffer(2) ]]
                             ,constant subFXVars& subVars[[buffer(3)]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
    vec4 texColor = getColor(inputTex0, input.tex1);
    vec4 burnColor = vars.burnColor;

#ifndef PRE_MULT
    multiplyAlpha(texColor);
    multiplyAlpha(burnColor);
#endif

    vec4 colorA = mix(vec4(0,0,0,0), texColor, u.mix);
    vec4 colorB = mix(texColor, vec4(0,0,0,0), u.mix);
    vec4 outColor = mix(colorB + u.transitionProgress * burnColor,
                        colorA + (1.0 - u.transitionProgress) * burnColor,
                        u.transitionProgress);

#ifdef OPACITY_MASK
    float maxOpacity = texColor.a;
    outColor = min(outColor, vec4(maxOpacity));
#endif

    multiplyOpacity(outColor, u.baseAlpha);

#ifdef USE_MASK
    multiplyOpacity(outColor, float(inputTexMask.sample(linearSampler, input.texMask).a));
#endif

#ifndef PRE_MULT
    divideAlpha(outColor);
#endif

    return outColor;
}
