#include "dxShaderIncludes.h"

static const float speed=1.;
static const float angle=2.;
static const float power=2.;



typedef struct
{
	float4 position: SV_POSITION;
	float2 tex1: TEXCOORD0;
	float2 tex2: TEXCOORD1;
	float2 texMask: TEXCOORD2;
} vertexOut2;


vertexOut2 vertexShader(txShaderVerts input)
{
	vertexOut2 output;
	output.position = mul(u.worldViewProj,float4(input.position,1));
	output.tex1 = input.tex1;
	output.tex2 = input.tex2;
	output.texMask = input.texMask;
	return output;
}

float4 pixelShader(vertexOut2 input) : SV_TARGET
{
	float fsMix;
	if(u.transitionProgress<=.5)
		fsMix=u.transitionProgress*2.;
	else
		fsMix=(1.-u.transitionProgress)*2.;

	vec2 p = input.tex1;
	vec2 p2 = input.tex2;

	float t = pow(u.transitionProgressLinear, power)*speed;
	p = p -0.5;
	for (int i = 0; i < 7; i++)
	{
		p = vec2(sin(t)*p.x + cos(t)*p.y, sin(t)*p.y - cos(t)*p.x);
		t += angle;
		p = abs(mod(p, 2.0) - 1.0);
	}
	abs(mod(p, 1.0));
	vec4 colorA=getColor(inputTex0,input.tex1);
	vec4 colorB=getColor(inputTex1,input.tex2);
	vec4 colorA2=getColor(inputTex0,p);
	vec4 colorB2=getColor(inputTex1,p2);
	colorA=mix(colorA,colorA2,1.0 - 2.0*abs(u.transitionProgressLinear - 0.5));
	colorB=mix(colorB,colorB2,1.0 - 2.0*abs(u.transitionProgressLinear - 0.5));
	float4 outColor=mix(colorB,colorA,u.transitionProgressLinear);

	#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,input.texMask).a);
	#endif


	return outColor;
}
